@@ -58,23 +58,25 @@ class JSONDecoder(json.JSONDecoder):
 
 class Session(object):
 
-	__version__ = "1.2.5-2"
+	__version__ = "1.3.1-1"
 
-	def __init__(self, domain=None, username=None, api_key=None, pwd=None, hostname=None, port=None, proxy=None, verbose=False, dev=False):
+	def __init__(self, domain=None, username=None, api_key=None, pwd=None, hostname=None, port=None, proxy=None, verbose=False, pretty_json=False, dev=False, path_logfile=None):
 		''' Setup; store credentials, authenticate, get a session key '''
 		# Generate a session ID
 		self._session_id = str(uuid.uuid4())
 		self._session_key = None # Have no session ID yet
 		self._verbose = verbose
+		self._pretty_json = pretty_json
 		self._proxy = proxy
 		self._dev = dev
+		Session._p_logfile = path_logfile
 		self._clearance = CLEARANCE_NONE
 		self.verbose("Creating Accsyn Python API session (v%s)"%Session.__version__)
 		for key in os.environ:
 			if key.startswith("FILMHUB_"):
 				Session.warning("Found old FilmHUB product environment variable '%s', please migrate!"%key)
 		if domain is None:
-			if not ('ACCSYN_DOMAIN' in os.environ or 'ACCSYN_ORG' in os.environ or 'FILMUB_DOMAIN' in os.environ or 'FILMHUB_ORG' in os.environ):
+			if not ('ACCSYN_DOMAIN' in os.environ or 'ACCSYN_ORG' in os.environ or 'FILMHUB_DOMAIN' in os.environ or 'FILMHUB_ORG' in os.environ):
 				raise AccsynException("Please supply your Accsyn domain/organization or set ACCSYN_DOMAIN environment!")
 		self._domain = domain or (os.environ['ACCSYN_DOMAIN'] if 'ACCSYN_DOMAIN' in os.environ else os.environ.get('ACCSYN_ORG', os.environ.get('FILMHUB_DOMAIN', os.environ.get('FILMHUB_ORG'))))
 		if username is None:
@@ -111,19 +113,38 @@ class Session(object):
 
 	@staticmethod
 	def info(s, standout=False):
-		if standout:
-			logging.info("-"*80)
-		logging.info(s)
-		if standout:
-			logging.info("-"*80)
+		if Session._p_logfile:
+			with open(Session._p_logfile, "a") as f:
+				try:
+					if standout:
+						f.write("-"*80+"\n")
+					f.write(s+"\n")
+					if standout:
+						f.write("-"*80+"\n")
+				finally:
+					f.flush()
+		else:
+			if standout:
+				logging.info("-"*80)
+			logging.info(s)
+			if standout:
+				logging.info("-"*80)
 
 	@staticmethod
 	def warning(s, standout=True):
-		if standout:
-			logging.warning("-"*80)
-		logging.warning(s)
-		if standout:
-			logging.warning("-"*80)
+		if Session._p_logfile:
+			with open(Session._p_logfile, "a") as f:
+				if standout:
+					f.write("[WARNING]"+"-"*80+"\n")
+				f.write("[WARNING]"+s+"\n")
+				if standout:
+					f.write("[WARNING]"+"-"*80+"\n")
+		else:
+			if standout:
+				logging.warning("-"*80)
+			logging.warning(s)
+			if standout:
+				logging.warning("-"*80)
 
 	def verbose(self, s):
 		if self._verbose:
@@ -330,7 +351,7 @@ class Session(object):
 		for iteration in range(0,2):
 			t_start = int(round(time.time() * 1000))
 			try:
-				self.verbose("REST %s %s, data: %s"%(method, url, data))
+				self.verbose("REST %s %s, data: %s"%(method, url, data if not self._pretty_json else Session.str(data)))
 				if method.lower() == "get":
 					r = requests.get(url, params = urllib.parse.quote(Session.safe_dumps(data)), timeout=(CONNECT_TO, READ_TO), verify=False, headers=headers)
 				elif method.lower() == "put":
@@ -356,12 +377,11 @@ class Session(object):
 				#retval = Accsyn.prepare_rest_deserialize(r.json())
 				retval = json.loads(r.text, cls=JSONDecoder)
 				if not quiet:
-					self.verbose("%s/%s REST %s result of %s: %s (~%sms)"%(
+					self.verbose("%s/%s REST %s result: %s (~%sms)"%(
 						hostname, 
 						uri, 
 						method, 
-						Session.obscure_dict_string(Session.safely_printable(str(data)).replace("'",'"')), 
-						Session.obscure_dict_string(Session.safely_printable(str(retval)).replace("'",'"')), 
+						Session.obscure_dict_string(Session.safely_printable(str(retval) if not self._pretty_json else Session.str(retval)).replace("'",'"')), 
 						t_start-t_end+1))
 				do_retry = False
 				if not retval.get('message') is None:
@@ -466,9 +486,10 @@ class Session(object):
 
 	# Create
 
-	def create(self, entitytype, data, entitytype_id=None):
+	def create(self, entitytype, data, entitytype_id=None, allow_duplicates=True):
 		''' Create an entity '''
 		assert (0<len((entitytype or "").strip())),("You must provide the entity type!")		
+		entitytype = entitytype.lower().strip()
 		if isinstance(data, str) or isinstance(data, str):
 			assert (0<len((data or "").strip())),("You must provide the data to create!")
 			# Is it JSON as a string or JSON in a file pointed to?
@@ -481,8 +502,11 @@ class Session(object):
 				else:
 					raise AccsynException("Cannot build JSON payload data, not a valid JSON string or path to a JSON file!")
 		else:
+			if isinstance(data, list):
+				data = {'tasks':data}
 			assert (not data is None and 0<len(data)),("Empty create data submitted!")
-
+		if entitytype in ["job","task"]:
+			data['allow_duplicates'] = allow_duplicates
 		d = self.event("POST", "%s/create"%Session.get_base_uri(entitytype), data, query=entitytype_id)
 		if d:
 			if 'result' in d:
@@ -492,9 +516,9 @@ class Session(object):
 
 	# Query
 
-	def find(self, query, attributes=None, finished=None, limit=None, skip=None):
+	def find(self, query, attributes=None, finished=None, limit=None, skip=None, create=False, update=False):
 		''' Return a list of something '''
-		assert (0<len(query or "") and (isinstance(query, str) or isinstance(query, str)  or isinstance(query, unicode))),("Invalid query type supplied, must be of string type!")
+		assert (0<len(query or "") and (isinstance(query, str) or isinstance(query, str) or isinstance(query, unicode))),("Invalid query type supplied, must be of string type!")
 		retval = None
 		d = self.decode_query(query)
 		data = {}
@@ -509,7 +533,7 @@ class Session(object):
 			parts = d['expression'].split("=")
 			assert (len(parts)==2 and parts[0].strip() == "entitytype"),("Please query attributes by expressions on the form 'attributes WHERE entitytype=job'")
 			entitytype = parts[1].strip().replace("'","").replace('"',"")
-			d = self.event("GET", "attributes", {'entitytype':entitytype})
+			d = self.event("GET", "attributes", {'entitytype':entitytype, 'create':create, 'update':update})
 			if d:
 				retval = d['result']
 		else:
@@ -583,13 +607,14 @@ class Session(object):
 			return response['result']
 
 	# File operations
-	def ls(self, p, recursive=False, maxdepth=None, files_only=False, directories_only=False):
+	def ls(self, p, recursive=False, maxdepth=None, getsize=False, files_only=False, directories_only=False):
 		assert (0<len(p or "") and (isinstance(p, str) or isinstance(p, str) or isinstance(p, dict) or isinstance(p, list))),("No path supplied, or not a string/list!")
 		data = {
 			'op':"ls",
 			'path':p,
 			'download':True,
-			'recursive':recursive
+			'recursive':recursive,
+			'getsize':getsize
 		}
 		if maxdepth:
 			data['maxdepth'] = maxdepth
